import json
import requests


class OpenRouter:
    def post_api_request(api_token, git_status, git_diff, model):
        spec = requests.get("https://github.com/conventional-commits/conventionalcommits.org/raw/refs/heads/master/content/v1.0.0/index.md")
        response = requests.post(
            url="https://openrouter.ai/api/v1/chat/completions",
            headers={
                "Authorization": f"Bearer {api_token}",
                "Content-Type": "application/json",
            },
            data=json.dumps({
                "model": model,
                "stream": False,
                "messages": [
                    {
                        "role": "system",
                        "content": "You are a git commit message generator. Your job is to generate conventional commit messages based on the provided git repository diff and file changes. You are to format the commit messages according to the specification provided in the following message. Interperet the following message as markdown.",
                    },
                    {
                        "role": "system",
                        "content": spec.text,
                    },
                    {
                        "role": "user",
                        "content": f"Generate a commit message for these changes:\n\nFile changes:\n{git_status}\n\nDiff:\n{git_diff}\n\nImportant:\n- Subject line should be 50-70 characters\n- Use imperative mood in subject line\n- Do not end subject line with period\n- Body should explain what and why, not how\n- For minor changes, use fix instead of feat\n\nResponse should be the commit message only, no explanations\n- If the commit is an initial commit, type is optional",
                    }
                ],
            })
        )
        if not response.ok:
            error_data = response.json()
            raise Exception(
                f"API request failed: {error_data.get('error', response.text)}"
            )
        else:
            return response



import argparse
import os
import subprocess
import tempfile
import textwrap

import colorama
from colorama import Fore, Style

from .config import Config
from .git_utils import GitUtils
from .open_router import OpenRouter

colorama.init()


def yes_no(prompt):
    while True:
        response = input(prompt + " (y/n): ").lower().strip()
        if response in ['y', 'yes']:
            return True
        elif response in ['n', 'no']:
            return False
        print("Please enter 'y' or 'n'")


def edit_message(msg):
    editor = os.environ.get('EDITOR', 'nano')
    temp = tempfile.NamedTemporaryFile(prefix='acmsg_', delete=False, mode='w')
    temp.write(msg)
    temp.close()
    subprocess.run([editor, temp.name])
    with open(temp.name, 'r') as temp:
        return temp.read()


def format_message(msg):
    lines = msg.splitlines()
    formatted_lines = []
    for line in lines:
        if len(line) > 80:
            wrapped = textwrap.wrap(line, 80)
            formatted_lines.extend(wrapped)
        else:
            formatted_lines.append(line)
    return '\n'.join(formatted_lines)


def print_message(message):
    print(f"\n{Fore.LIGHTBLACK_EX}Commit message:{Style.RESET_ALL}\n")
    print(f"  {message.replace('\n', '\n  ')}\n")


def prompt_for_action(message):
    prompt = (Fore.LIGHTBLACK_EX + "Commit with this message? (y/n/e[dit]): " + Style.RESET_ALL)
    while True:
        opt = input(prompt).lower().strip()
        match opt:
            case "e" | "edit":
                message = edit_message(message)
                formatted_message = format_message(message)
                print_message(formatted_message)
            case "n" | "no":
                return False
            case "y" | "yes":
                return True
            case _:
                print(f"{Fore.RED}Invalid option: {opt}{Style.RESET_ALL}")


def handle_commit(args):
    if not GitUtils.is_git_repo():
        print(Fore.RED + "Error: not a git repository" + Style.RESET_ALL)
        exit(1)

    cfg = Config()
    api_token = cfg.api_token
    model = args.model if args.model else cfg.model
    git_status = GitUtils.git_status()
    git_diff = GitUtils.git_diff()

    if not git_status:
        print(Fore.YELLOW + "Nothing to commit" + Style.RESET_ALL)
        exit(1)
    if not git_diff:
        print(Fore.YELLOW + "Nothing to commit" + Style.RESET_ALL)
        exit(1)

    try:
        response = OpenRouter.post_api_request(
            api_token, git_status, git_diff, model)

        message = response.json()['choices'][0]['message']['content']
        formatted_message = format_message(message)
        print_message(formatted_message)

        if prompt_for_action(formatted_message):
            try:
                GitUtils.git_commit(formatted_message)
                print("Commit successful!")
            except Exception as e:
                print(f"Error committing: {e}")
                return False
        else:
            print("Commit cancelled")
    except Exception as e:
        print(f"Error: {e}")
        exit(1)


def handle_config(args):
    cfg = Config()
    if args.token:
        cfg.set_param(param='api_token', value=args.token)
        print("API token configuration saved.")
    elif args.model:
        cfg.set_param(param='model', value=args.model)
        print("Model configuration saved.")
    elif args._:
        print("Error: invalid configuration option")
    else:
        return True


def main():
    parser = argparse.ArgumentParser(prog='acmsg')
    subparsers = parser.add_subparsers(dest='command', help='Commands')

    commit_parser = subparsers.add_parser('commit', help='Generate commit message')
    commit_parser.add_argument('--model', type=str, help='Model to use for generation')

    config_parser = subparsers.add_parser('config', help='Configuration commands')
    config_subparsers = config_parser.add_subparsers(dest='config_command')

    config_set = config_subparsers.add_parser('set', help='Set configuration values')
    config_set.add_argument('param', choices=['api_token', 'model'])
    config_set.add_argument('value', type=str, help="Parameter value")

    config_get = config_subparsers.add_parser('get', help='Get configuration values')
    config_get.add_argument('param', choices=['api_token', 'model'])

    subparsers.add_parser('help', help='Show help')

    args = parser.parse_args()

    if args.command == 'help' or not args.command:
        parser.print_help()
        exit(0)

    if args.command == 'commit':
        handle_commit(args)
    elif args.command == 'config':
        if args.config_command == 'set':
            if args.param == 'api_token':
                args.token = args.value
            elif args.param == 'model':
                args.model = args.value
            handle_config(args)
        elif args.config_command == 'get':
            cfg = Config()
            if args.param == 'api_token':
                param_value = cfg.get_param('api_token')
            elif args.param == 'model':
                param_value = cfg.get_param('model')
            print(param_value)


if __name__ == "__main__":
    main()
    exit(0)



import subprocess


class GitUtils:
    def is_git_repo():
        output = subprocess.run(
            ["git", "rev-parse", "--is-inside-work-tree"],
            capture_output=True, text=True, check=True)
        return output.stdout.strip()

    def git_status():
        git_status = subprocess.run(
            ["git", "diff", "--cached", "--name-status"],
            capture_output=True, text=True)
        if git_status.returncode != 0:
            raise Exception(f"Git status failed:\n{git_status.stderr}")
        else:
            return git_status.stdout.strip().replace('\t', ' ').split('\n')

    def git_diff():
        diff = subprocess.run(
            ["git", "diff", "--cached"],
            capture_output=True, text=True)
        return diff.stdout.strip()

    def git_commit(message):
        commit = subprocess.run(
            ["git", "commit", "-m", message],
            capture_output=True, text=True)
        if commit.returncode != 0:
            raise Exception(commit.stderr)
        else:
            return commit.stdout.strip()



import os
import yaml


class Config:
    def __init__(self):
        self.config_dir = self.create_dir_if_not_exists()
        self.config_file = self.create_file_if_not_exists()
        self.model = self.get_param("model")
        self.api_token = self.get_param('api_token')

    def create_dir_if_not_exists(self):
        home = os.getenv("HOME") or os.path.expanduser("~")
        config_dir = f"{home}/.config/acmsg"
        os.makedirs(config_dir, exist_ok=True)
        return config_dir

    def create_file_if_not_exists(self):
        config_file = f"{self.config_dir}/config.yaml"
        data = {"api_token": "", "model": "deepseek/deepseek-r1:free"}

        if not os.path.exists(config_file):
            os.makedirs(self.config_dir, exist_ok=True)
            with open(config_file, "w") as f:
                yaml.dump(data, f)

        return config_file

    def set_param(self, param, value):
        config_file = self.config_file
        with open(config_file, "r") as f:
            data = yaml.safe_load(f)

        data[param] = value

        with open(config_file, "w") as f:
            yaml.dump(data, f, default_flow_style=False)

    def get_param(self, param):
        config_file = self.config_file

        with open(config_file, "r") as f:
            data = yaml.safe_load(f)

        return data.get(param)
